from vcCommand import *
import string
import re 


RESET_DYNAMIC_PROPERTY_UNIT = False
SEPARATORS = " ()+-/*:"

def findtoken2(expr, str):
  #print "EXPR = ",expr
  s = re.sub(str, 'RD.', expr) 
  #print "RESULT ",s 
  if s != "":
    return True
  else:
    return False

# Manage following type of cases expression token matching
# "Sx(Foo/1.5)"
# "CubeLength"
# "Foo::Bar"
   
def findtoken(expr, str):
  i = expr.find(str)
  try:
    if i != -1:
      #print expr, str
      start_ok = False
      end_ok = False
      # Check both ends to validate keyword
      if i>0:
        start = expr[i-1]
        start_ok = SEPARATORS.find(start) != -1
      else:
        start_ok = True
        
      l = len(expr)
      if (i+len(str))<l:
        end = expr[i+len(str)]
        end_ok = SEPARATORS.find(end) != -1
      else:
        end_ok = True
      #print start_ok, end_ok
      if start_ok and end_ok:
        return True
  except:
    return False
  return False



def checkPropertyUsage(node, name):
  root = node.RootFeature
  for c in root.Children:
    if checkPropertyFeatureUsage(c, name):
      return True
  return False
    
    
""" 
  Features to analyze
VC_ANGULARCLONE
VC_BLOCK
VC_CONE
VC_CYLINDER
VC_EXTRUDE
VC_FRAME
VC_GEOMETRY
VC_LINEARCLONE
VC_MIRROR
VC_PLANE
VC_PYTHONFEATURE
VC_REVOLVE
VC_SPHERE
VC_SWITCH
VC_TRANSFORM
VC_WEDGE
"""
def checkPropertyFeatureUsage(feat, name):
  if feat.Type == VC_ANGULARCLONE:
    if checkExpressionProperties([feat.Count,feat.Step,feat.Offset] , name):
      return True

  if feat.Type == VC_BLOCK:
    if checkExpressionProperties([feat.Length,feat.Width,feat.Height], name):
      return True
      
  if feat.Type == VC_CONE:
    if checkExpressionProperties([feat.BottomRadius, feat.TopRadius, feat.Height, feat.StartSweep, feat.EndSweep, feat.Sections], name):
      return True
      
  if feat.Type == VC_CYLINDER:
    if checkExpressionProperties([feat.Radius, feat.Height, feat.StartSweep, feat.EndSweep, feat.Sections], name):
      return True
      
  if feat.Type == VC_EXTRUDE:
    if checkExpressionProperties([feat.Direction,feat.Length] , name):
      return True
       
  if feat.Type == VC_LINEARCLONE:
    if checkExpressionProperties([feat.Count,feat.Step,feat.Direction] , name):
      return True
      
  if feat.Type == VC_MIRROR:
    if checkExpressionProperties([feat.Direction] , name):
      return True
      
  if feat.Type == VC_PLANE:
    if checkExpressionProperties([feat.Size] , name):
      return True
      
  if feat.Type == VC_REVOLVE:
    if checkExpressionProperties([feat.Sections, feat.RevolveAngle, feat.RevolveAxis, feat.RevolveOffset] , name):
      return True
      
  if feat.Type == VC_SPHERE:
    if checkExpressionProperties([feat.Radius, feat.StartSweep, feat.EndSweep, feat.Sections, feat.Spans] , name):
      return True
      
  if feat.Type == VC_TRANSFORM:
    if checkExpressionProperties([feat.Expression], name):
      return True
      
  if feat.Type == VC_SWITCH:
    if checkExpressionProperties([feat.Choice], name):
      return True
      
  if feat.Type == VC_WEDGE:
    if checkExpressionProperties([feat.BiggerHeight, feat.BiggerLength, feat.SmallerHeight, feat.SmallerLength, feat.Width], name):
      return True
      
  for c in feat.Children:
    if checkPropertyFeatureUsage(c, name):
      return True
  return False
  
  
def checkExpressionProperties(exprs, name):
  for expr in exprs:
    if findtoken(expr,name):
      return True
  return False
 
 
def AnalyzeAndFixProperties(autofix):
  
  #Notify the user with a message box to continue or not
  app = getApplication()
  ret = app.messageBox("Note: this function will optimise the Rebuild of components properties. In some cases, it might make some unwanted changes to your simulation. Please remember to check your simulation after running this function. Do you want to continue?",
  VC_MESSAGE_TYPE_QUESTION,VC_MESSAGE_BUTTONS_YESNO)
  
  if not ret == VC_MESSAGE_RESULT_YES:
      print "Cancelled"
      return
     
  warn_count = 0
  warn_dynamic_count = 0
  for comp in app.Components:
    #print "Analyzing component \'",comp.Name,"\' properties"
    # Check all component properties
    for p in comp.Properties:
      if checkPropertyUsage(comp, p.Name):
        if not p.ChangeOnRebuild:
          #warn_count += 1
          #print "WARNING: Property \'",p.Name,"\' requires rebuild option"
          p.ChangeOnRebuild = True
      else:
        if p.ChangeOnRebuild:
          warn_count += 1
          print "OPTIMIZED: Inside the Component '{}', Property '{}' does not require rebuild option (unoptimal)".format(comp.Name, p.Name)
          p.ChangeOnRebuild = False
    # Check creator behaviour template components
    creators = comp.findBehavioursByType(VC_COMPONENTCREATOR)
    for cre in creators:
      template = cre.TemplateComponent
      if template:
        #print "Analyzing template component \'",template.Name,"\' properties at component \'",comp.Name,"\'"
        for p in template.Properties:
          # Set dynamic property quantities to empty as they require heavy copying burden
          if RESET_DYNAMIC_PROPERTY_UNIT:
            try:
              p.Quantity = None
            except:
              pass
          if checkPropertyUsage(template, p.Name):
            if not p.ChangeOnRebuild:
              #warn_dynamic_count += 1
              #print "WARNING: Property \'",template.Name,"\' requires rebuild option"
              p.ChangeOnRebuild = True
          else:
            if p.ChangeOnRebuild:
              warn_dynamic_count += 1
              print "OPTIMIZED: Inside the Component '{}', Property '{}' does not require rebuild option (unoptimal)".format(template.Name, p.Name)
              p.ChangeOnRebuild = False

  print "-" *50
  print "Analyzed ",len(app.Components)," components"
  print "Total property optimization with Rebuild fixes at static components ",warn_count
  print "Total property optimization with Rebuild fixes at dynamic components ",warn_dynamic_count
  print "-" *50
  
def first_state():
  AnalyzeAndFixProperties(True)


# Add all command states
addState( first_state )


